@@ -1,17 +1,27 @@
 /*
-    Copyright (c) 2007-2013 Contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2015 Contributors as noted in the AUTHORS file
 
-    This file is part of 0MQ.
+    This file is part of libzmq, the ZeroMQ core engine in C++.
 
-    0MQ is free software; you can redistribute it and/or modify it under
-    the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 3 of the License, or
+    libzmq is free software; you can redistribute it and/or modify it under
+    the terms of the GNU Lesser General Public License (LGPL) as published
+    by the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
-    0MQ is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
+    As a special exception, the Contributors give you permission to link
+    this library with independent modules to produce an executable,
+    regardless of the license terms of these independent modules, and to
+    copy and distribute the resulting executable under terms of your choice,
+    provided that you also meet, for each linked independent module, the
+    terms and conditions of the license of that module. An independent
+    module is a module which is not derived from or based on this library.
+    If you modify this library, you must extend this exception to your
+    version of the library.
+
+    libzmq is distributed in the hope that it will be useful, but WITHOUT
+    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
+    License for more details.
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
@@ -39,6 +49,7 @@
 #include "socket_base.hpp"
 #include "tcp_listener.hpp"
 #include "ipc_listener.hpp"
+#include "tipc_listener.hpp"
 #include "tcp_connecter.hpp"
 #include "io_thread.hpp"
 #include "session_base.hpp"
@@ -52,6 +63,7 @@
 #include "address.hpp"
 #include "ipc_address.hpp"
 #include "tcp_address.hpp"
+#include "tipc_address.hpp"
 #ifdef ZMQ_HAVE_OPENPGM
 #include "pgm_socket.hpp"
 #endif
@@ -119,10 +131,11 @@ zmq::socket_base_t *zmq::socket_base_t::create (int type_, class ctx_t *parent_,
             errno = EINVAL;
             return NULL;
     }
-    if (s->mailbox.get_fd () == -1)
-        return NULL;
 
     alloc_assert (s);
+    if (s->mailbox.get_fd () == retired_fd)
+        return NULL;
+
     return s;
 }
 
@@ -134,6 +147,7 @@ zmq::socket_base_t::socket_base_t (ctx_t *parent_, uint32_t tid_, int sid_) :
     last_tsc (0),
     ticks (0),
     rcvmore (false),
+    file_desc(-1),
     monitor_socket (NULL),
     monitor_events (0)
 {
@@ -174,7 +188,7 @@ int zmq::socket_base_t::parse_uri (const char *uri_,
     }
     protocol_ = uri.substr (0, pos);
     address_ = uri.substr (pos + 3);
-    
+
     if (protocol_.empty () || address_.empty ()) {
         errno = EINVAL;
         return -1;
@@ -185,12 +199,16 @@ int zmq::socket_base_t::parse_uri (const char *uri_,
 int zmq::socket_base_t::check_protocol (const std::string &protocol_)
 {
     //  First check out whether the protcol is something we are aware of.
-    if (protocol_ != "inproc" && protocol_ != "ipc" && protocol_ != "tcp" &&
-          protocol_ != "pgm" && protocol_ != "epgm") {
+    if (protocol_ != "inproc"
+    &&  protocol_ != "ipc"
+    &&  protocol_ != "tcp"
+    &&  protocol_ != "pgm"
+    &&  protocol_ != "epgm"
+    &&  protocol_ != "tipc"
+    &&  protocol_ != "norm") {
         errno = EPROTONOSUPPORT;
         return -1;
     }
-
     //  If 0MQ is not compiled with OpenPGM, pgm and epgm transports
     //  are not avaialble.
 #if !defined ZMQ_HAVE_OPENPGM
@@ -200,6 +218,13 @@ int zmq::socket_base_t::check_protocol (const std::string &protocol_)
     }
 #endif
 
+#if !defined ZMQ_HAVE_NORM
+    if (protocol_ == "norm") {
+        errno = EPROTONOSUPPORT;
+        return -1;
+    }
+#endif // !ZMQ_HAVE_NORM
+
     //  IPC transport is not available on Windows and OpenVMS.
 #if defined ZMQ_HAVE_WINDOWS || defined ZMQ_HAVE_OPENVMS
     if (protocol_ == "ipc") {
@@ -209,10 +234,18 @@ int zmq::socket_base_t::check_protocol (const std::string &protocol_)
     }
 #endif
 
+    // TIPC transport is only available on Linux.
+#if !defined ZMQ_HAVE_TIPC
+    if (protocol_ == "tipc") {
+        errno = EPROTONOSUPPORT;
+        return -1;
+    }
+#endif
+
     //  Check whether socket type and transport protocol match.
     //  Specifically, multicast protocols can't be combined with
     //  bi-directional messaging patterns (socket types).
-    if ((protocol_ == "pgm" || protocol_ == "epgm") &&
+    if ((protocol_ == "pgm" || protocol_ == "epgm" || protocol_ == "norm") &&
           options.type != ZMQ_PUB && options.type != ZMQ_SUB &&
           options.type != ZMQ_XPUB && options.type != ZMQ_XSUB) {
         errno = ENOCOMPATPROTO;
@@ -228,7 +261,7 @@ void zmq::socket_base_t::attach_pipe (pipe_t *pipe_, bool subscribe_to_all_)
     //  First, register the pipe so that we can terminate it later on.
     pipe_->set_event_sink (this);
     pipes.push_back (pipe_);
-    
+
     //  Let the derived socket type know about new pipe.
     xattach_pipe (pipe_, subscribe_to_all_);
 
@@ -332,27 +365,22 @@ int zmq::socket_base_t::bind (const char *addr_)
     //  Parse addr_ string.
     std::string protocol;
     std::string address;
-    rc = parse_uri (addr_, protocol, address);
-    if (rc != 0)
-        return -1;
-
-    rc = check_protocol (protocol);
-    if (rc != 0)
+    if (parse_uri (addr_, protocol, address) || check_protocol (protocol))
         return -1;
 
     if (protocol == "inproc") {
-        endpoint_t endpoint = {this, options};
-        int rc = register_endpoint (addr_, endpoint);
+        const endpoint_t endpoint = { this, options };
+        const int rc = register_endpoint (addr_, endpoint);
         if (rc == 0) {
-            connect_pending(addr_, this);
+            connect_pending (addr_, this);
             last_endpoint.assign (addr_);
         }
         return rc;
     }
 
-    if (protocol == "pgm" || protocol == "epgm") {
+    if (protocol == "pgm" || protocol == "epgm" || protocol == "norm") {
         //  For convenience's sake, bind can be used interchageable with
-        //  connect for PGM and EPGM transports.
+        //  connect for PGM, EPGM and NORM transports.
         return connect (addr_);
     }
 
@@ -378,7 +406,7 @@ int zmq::socket_base_t::bind (const char *addr_)
         // Save last endpoint URI
         listener->get_address (last_endpoint);
 
-        add_endpoint (addr_, (own_t *) listener, NULL);
+        add_endpoint (last_endpoint.c_str (), (own_t *) listener, NULL);
         return 0;
     }
 
@@ -397,6 +425,25 @@ int zmq::socket_base_t::bind (const char *addr_)
         // Save last endpoint URI
         listener->get_address (last_endpoint);
 
+        add_endpoint (last_endpoint.c_str (), (own_t *) listener, NULL);
+        return 0;
+    }
+#endif
+#if defined ZMQ_HAVE_TIPC
+    if (protocol == "tipc") {
+         tipc_listener_t *listener = new (std::nothrow) tipc_listener_t (
+              io_thread, this, options);
+         alloc_assert (listener);
+         int rc = listener->set_address (address.c_str ());
+         if (rc != 0) {
+             delete listener;
+             event_bind_failed (address, zmq_errno());
+             return -1;
+         }
+
+        // Save last endpoint URI
+        listener->get_address (last_endpoint);
+
         add_endpoint (addr_, (own_t *) listener, NULL);
         return 0;
     }
@@ -421,12 +468,7 @@ int zmq::socket_base_t::connect (const char *addr_)
     //  Parse addr_ string.
     std::string protocol;
     std::string address;
-    rc = parse_uri (addr_, protocol, address);
-    if (rc != 0)
-        return -1;
-
-    rc = check_protocol (protocol);
-    if (rc != 0)
+    if (parse_uri (addr_, protocol, address) || check_protocol (protocol))
         return -1;
 
     if (protocol == "inproc") {
@@ -448,7 +490,8 @@ int zmq::socket_base_t::connect (const char *addr_)
         int rcvhwm = 0;
         if (peer.socket == NULL)
             rcvhwm = options.rcvhwm;
-        else if (options.rcvhwm != 0 && peer.options.sndhwm != 0)
+        else
+        if (options.rcvhwm != 0 && peer.options.sndhwm != 0)
             rcvhwm = options.rcvhwm + peer.options.sndhwm;
 
         //  Create a bi-directional pipe to connect the peers.
@@ -470,17 +513,26 @@ int zmq::socket_base_t::connect (const char *addr_)
         //  Attach local end of the pipe to this socket object.
         attach_pipe (new_pipes [0]);
 
-        if (!peer.socket)
-        {
-            endpoint_t endpoint = {this, options};
-            pending_connection_t pending_connection = {endpoint, new_pipes [0], new_pipes [1]};
-            pend_connection (addr_, pending_connection);
+        if (!peer.socket) {
+            //  The peer doesn't exist yet so we don't know whether
+            //  to send the identity message or not. To resolve this,
+            //  we always send our identity and drop it later if
+            //  the peer doesn't expect it.
+            msg_t id;
+            rc = id.init_size (options.identity_size);
+            errno_assert (rc == 0);
+            memcpy (id.data (), options.identity, options.identity_size);
+            id.set_flags (msg_t::identity);
+            bool written = new_pipes [0]->write (&id);
+            zmq_assert (written);
+            new_pipes [0]->flush ();
+
+            const endpoint_t endpoint = {this, options};
+            pend_connection (std::string (addr_), endpoint, new_pipes);
         }
-        else
-        {
+        else {
             //  If required, send the identity of the local socket to the peer.
             if (peer.options.recv_identity) {
-    
                 msg_t id;
                 rc = id.init_size (options.identity_size);
                 errno_assert (rc == 0);
@@ -513,10 +565,22 @@ int zmq::socket_base_t::connect (const char *addr_)
         last_endpoint.assign (addr_);
 
         // remember inproc connections for disconnect
-        inprocs.insert (inprocs_t::value_type (std::string (addr_), new_pipes[0]));
+        inprocs.insert (inprocs_t::value_type (std::string (addr_), new_pipes [0]));
 
         return 0;
     }
+    bool is_single_connect = (options.type == ZMQ_DEALER ||
+                              options.type == ZMQ_SUB ||
+                              options.type == ZMQ_REQ);
+    if (unlikely (is_single_connect)) {
+        const endpoints_t::iterator it = endpoints.find (addr_);
+        if (it != endpoints.end ()) {
+            // There is no valid use for multiple connects for SUB-PUB nor
+            // DEALER-ROUTER nor REQ-REP. Multiple connects produces
+            // nonsensical results.
+            return 0;
+        }
+    }
 
     //  Choose the I/O thread to run the session in.
     io_thread_t *io_thread = choose_io_thread (options.affinity);
@@ -530,14 +594,41 @@ int zmq::socket_base_t::connect (const char *addr_)
 
     //  Resolve address (if needed by the protocol)
     if (protocol == "tcp") {
-        paddr->resolved.tcp_addr = new (std::nothrow) tcp_address_t ();
-        alloc_assert (paddr->resolved.tcp_addr);
-        int rc = paddr->resolved.tcp_addr->resolve (
-            address.c_str (), false, options.ipv6);
-        if (rc != 0) {
+        //  Do some basic sanity checks on tcp:// address syntax
+        //  - hostname starts with digit or letter, with embedded '-' or '.'
+        //  - IPv6 address may contain hex chars and colons.
+        //  - IPv4 address may contain decimal digits and dots.
+        //  - Address must end in ":port" where port is *, or numeric
+        //  - Address may contain two parts separated by ':'
+        //  Following code is quick and dirty check to catch obvious errors,
+        //  without trying to be fully accurate.
+        const char *check = address.c_str ();
+        if (isalnum (*check) || isxdigit (*check)) {
+            check++;
+            while (isalnum  (*check)
+                || isxdigit (*check)
+                || *check == '.' || *check == '-' || *check == ':'|| *check == ';')
+                check++;
+        }
+        //  Assume the worst, now look for success
+        rc = -1;
+        //  Did we reach the end of the address safely?
+        if (*check == 0) {
+            //  Do we have a valid port string? (cannot be '*' in connect
+            check = strrchr (address.c_str (), ':');
+            if (check) {
+                check++;
+                if (*check && (isdigit (*check)))
+                    rc = 0;     //  Valid
+            }
+        }
+        if (rc == -1) {
+            errno = EINVAL;
             delete paddr;
             return -1;
         }
+        //  Defer resolution until a socket is opened
+        paddr->resolved.tcp_addr = NULL;
     }
 #if !defined ZMQ_HAVE_WINDOWS && !defined ZMQ_HAVE_OPENVMS
     else
@@ -551,6 +642,9 @@ int zmq::socket_base_t::connect (const char *addr_)
         }
     }
 #endif
+
+// TBD - Should we check address for ZMQ_HAVE_NORM???
+
 #ifdef ZMQ_HAVE_OPENPGM
     if (protocol == "pgm" || protocol == "epgm") {
         struct pgm_addrinfo_t *res = NULL;
@@ -562,14 +656,27 @@ int zmq::socket_base_t::connect (const char *addr_)
             return -1;
     }
 #endif
+#if defined ZMQ_HAVE_TIPC
+    else
+    if (protocol == "tipc") {
+        paddr->resolved.tipc_addr = new (std::nothrow) tipc_address_t ();
+        alloc_assert (paddr->resolved.tipc_addr);
+        int rc = paddr->resolved.tipc_addr->resolve (address.c_str());
+        if (rc != 0) {
+            delete paddr;
+            return -1;
+        }
+    }
+#endif
+
     //  Create session.
     session_base_t *session = session_base_t::create (io_thread, true, this,
         options, paddr);
     errno_assert (session);
 
     //  PGM does not support subscription forwarding; ask for all data to be
-    //  sent to this pipe.
-    bool subscribe_to_all = protocol == "pgm" || protocol == "epgm";
+    //  sent to this pipe. (same for NORM, currently?)
+    bool subscribe_to_all = protocol == "pgm" || protocol == "epgm" || protocol == "norm";
     pipe_t *newpipe = NULL;
 
     if (options.immediate != 1 || subscribe_to_all) {
@@ -609,7 +716,7 @@ void zmq::socket_base_t::add_endpoint (const char *addr_, own_t *endpoint_, pipe
 {
     //  Activate the session. Make it a child of this socket.
     launch_child (endpoint_);
-    endpoints.insert (endpoints_t::value_type (std::string (addr_), endpoint_pipe_t(endpoint_, pipe)));
+    endpoints.insert (endpoints_t::value_type (std::string (addr_), endpoint_pipe_t (endpoint_, pipe)));
 }
 
 int zmq::socket_base_t::term_endpoint (const char *addr_)
@@ -635,24 +742,21 @@ int zmq::socket_base_t::term_endpoint (const char *addr_)
     //  Parse addr_ string.
     std::string protocol;
     std::string address;
-    rc = parse_uri (addr_, protocol, address);
-    if (rc != 0)
-        return -1;
-
-    rc = check_protocol (protocol);
-    if (rc != 0)
+    if (parse_uri (addr_, protocol, address) || check_protocol (protocol))
         return -1;
 
     // Disconnect an inproc socket
     if (protocol == "inproc") {
+        if (unregister_endpoint (std::string (addr_), this) == 0)
+            return 0;
         std::pair <inprocs_t::iterator, inprocs_t::iterator> range = inprocs.equal_range (std::string (addr_));
         if (range.first == range.second) {
             errno = ENOENT;
             return -1;
         }
-    
+
         for (inprocs_t::iterator it = range.first; it != range.second; ++it)
-            it->second->terminate(true);
+            it->second->terminate (true);
         inprocs.erase (range.first, range.second);
         return 0;
     }
@@ -667,7 +771,7 @@ int zmq::socket_base_t::term_endpoint (const char *addr_)
     for (endpoints_t::iterator it = range.first; it != range.second; ++it) {
         //  If we have an associated pipe, terminate it.
         if (it->second.second != NULL)
-            it->second.second->terminate(false);
+            it->second.second->terminate (false);
         term_child (it->second.first);
     }
     endpoints.erase (range.first, range.second);
@@ -700,6 +804,8 @@ int zmq::socket_base_t::send (msg_t *msg_, int flags_)
     if (flags_ & ZMQ_SNDMORE)
         msg_->set_flags (msg_t::more);
 
+    msg_->reset_metadata ();
+
     //  Try to send the message.
     rc = xsend (msg_);
     if (rc == 0)
@@ -774,6 +880,8 @@ int zmq::socket_base_t::recv (msg_t *msg_, int flags_)
 
     //  If we have the message, return immediately.
     if (rc == 0) {
+        if (file_desc != retired_fd)
+            msg_->set_fd(file_desc);
         extract_flags (msg_);
         return 0;
     }
@@ -790,6 +898,8 @@ int zmq::socket_base_t::recv (msg_t *msg_, int flags_)
         rc = xrecv (msg_);
         if (rc < 0)
             return rc;
+        if (file_desc != retired_fd)
+            msg_->set_fd(file_desc);
         extract_flags (msg_);
         return 0;
     }
@@ -822,6 +932,8 @@ int zmq::socket_base_t::recv (msg_t *msg_, int flags_)
         }
     }
 
+    if (file_desc != retired_fd)
+        msg_->set_fd(file_desc);
     extract_flags (msg_);
     return 0;
 }
@@ -830,7 +942,7 @@ int zmq::socket_base_t::close ()
 {
     //  Mark the socket as dead
     tag = 0xdeadbeef;
-    
+
     //  Transfer the ownership of the socket from this application thread
     //  to the reaper thread which will take care of the rest of shutdown
     //  process.
@@ -877,7 +989,7 @@ int zmq::socket_base_t::process_commands (int timeout_, bool throttle_)
         //  commands recently, so that we can throttle the new commands.
 
         //  Get the CPU's tick counter. If 0, the counter is not available.
-        uint64_t tsc = zmq::clock_t::rdtsc ();
+        const uint64_t tsc = zmq::clock_t::rdtsc ();
 
         //  Optimised version of command processing - it doesn't have to check
         //  for incoming commands each time. It does so only if certain time
@@ -982,6 +1094,11 @@ int zmq::socket_base_t::xrecv (msg_t *)
     return -1;
 }
 
+zmq::blob_t zmq::socket_base_t::get_credential () const
+{
+    return blob_t ();
+}
+
 void zmq::socket_base_t::xread_activated (pipe_t *)
 {
     zmq_assert (false);
@@ -1060,12 +1177,11 @@ void zmq::socket_base_t::pipe_terminated (pipe_t *pipe_)
     xpipe_terminated (pipe_);
 
     // Remove pipe from inproc pipes
-    for (inprocs_t::iterator it = inprocs.begin(); it != inprocs.end(); ++it) {
+    for (inprocs_t::iterator it = inprocs.begin (); it != inprocs.end (); ++it)
         if (it->second == pipe_) {
-            inprocs.erase(it);
+            inprocs.erase (it);
             break;
         }
-    }    
 
     //  Remove the pipe from the list of attached pipes and confirm its
     //  termination if we are already shutting down.
@@ -1079,191 +1195,146 @@ void zmq::socket_base_t::extract_flags (msg_t *msg_)
     //  Test whether IDENTITY flag is valid for this socket type.
     if (unlikely (msg_->flags () & msg_t::identity))
         zmq_assert (options.recv_identity);
-  
+
     //  Remove MORE flag.
     rcvmore = msg_->flags () & msg_t::more ? true : false;
 }
 
 int zmq::socket_base_t::monitor (const char *addr_, int events_)
 {
-    int rc;
     if (unlikely (ctx_terminated)) {
         errno = ETERM;
         return -1;
     }
-
-    // Support deregistering monitoring endpoints as well
+    //  Support deregistering monitoring endpoints as well
     if (addr_ == NULL) {
         stop_monitor ();
         return 0;
     }
-
     //  Parse addr_ string.
     std::string protocol;
     std::string address;
-    rc = parse_uri (addr_, protocol, address);
-    if (rc != 0)
+    if (parse_uri (addr_, protocol, address) || check_protocol (protocol))
         return -1;
 
-    rc = check_protocol (protocol);
-    if (rc != 0)
-        return -1;
-
-    // Event notification only supported over inproc://
+    //  Event notification only supported over inproc://
     if (protocol != "inproc") {
         errno = EPROTONOSUPPORT;
         return -1;
     }
-
-    // Register events to monitor
+    //  Register events to monitor
     monitor_events = events_;
     monitor_socket = zmq_socket (get_ctx (), ZMQ_PAIR);
     if (monitor_socket == NULL)
         return -1;
 
-    // Never block context termination on pending event messages
+    //  Never block context termination on pending event messages
     int linger = 0;
-    rc = zmq_setsockopt (monitor_socket, ZMQ_LINGER, &linger, sizeof (linger));
+    int rc = zmq_setsockopt (monitor_socket, ZMQ_LINGER, &linger, sizeof (linger));
     if (rc == -1)
-         stop_monitor ();
+        stop_monitor ();
 
-    // Spawn the monitor socket endpoint
+    //  Spawn the monitor socket endpoint
     rc = zmq_bind (monitor_socket, addr_);
     if (rc == -1)
          stop_monitor ();
     return rc;
 }
 
-void zmq::socket_base_t::event_connected (std::string &addr_, int fd_)
+void zmq::socket_base_t::set_fd(zmq::fd_t fd_)
 {
-    if (monitor_events & ZMQ_EVENT_CONNECTED) {
-        zmq_event_t event;
-        event.event = ZMQ_EVENT_CONNECTED;
-        event.value = fd_;
-        monitor_event (event, addr_);
-    }
+    file_desc = fd_;
 }
 
-void zmq::socket_base_t::event_connect_delayed (std::string &addr_, int err_)
+zmq::fd_t zmq::socket_base_t::fd()
 {
-    if (monitor_events & ZMQ_EVENT_CONNECT_DELAYED) {
-        zmq_event_t event;
-        event.event = ZMQ_EVENT_CONNECT_DELAYED;
-        event.value = err_;
-        monitor_event (event, addr_);
-    }
+    return file_desc;
 }
 
-void zmq::socket_base_t::event_connect_retried (std::string &addr_, int interval_)
+void zmq::socket_base_t::event_connected (const std::string &addr_, int fd_)
 {
-    if (monitor_events & ZMQ_EVENT_CONNECT_RETRIED) {
-        zmq_event_t event;
-        event.event = ZMQ_EVENT_CONNECT_RETRIED;
-        event.value = interval_;
-        monitor_event (event, addr_);
-    }
+    if (monitor_events & ZMQ_EVENT_CONNECTED)
+        monitor_event (ZMQ_EVENT_CONNECTED, fd_, addr_);
 }
 
-void zmq::socket_base_t::event_listening (std::string &addr_, int fd_)
+void zmq::socket_base_t::event_connect_delayed (const std::string &addr_, int err_)
 {
-    if (monitor_events & ZMQ_EVENT_LISTENING) {
-        zmq_event_t event;
-        event.event = ZMQ_EVENT_LISTENING;
-        event.value = fd_;
-        monitor_event (event, addr_);
-    }
+    if (monitor_events & ZMQ_EVENT_CONNECT_DELAYED)
+        monitor_event (ZMQ_EVENT_CONNECT_DELAYED, err_, addr_);
 }
 
-void zmq::socket_base_t::event_bind_failed (std::string &addr_, int err_)
+void zmq::socket_base_t::event_connect_retried (const std::string &addr_, int interval_)
 {
-    if (monitor_events & ZMQ_EVENT_BIND_FAILED) {
-        zmq_event_t event;
-        event.event = ZMQ_EVENT_BIND_FAILED;
-        event.value = err_;
-        monitor_event (event, addr_);
-    }
+    if (monitor_events & ZMQ_EVENT_CONNECT_RETRIED)
+        monitor_event (ZMQ_EVENT_CONNECT_RETRIED, interval_, addr_);
 }
 
-void zmq::socket_base_t::event_accepted (std::string &addr_, int fd_)
+void zmq::socket_base_t::event_listening (const std::string &addr_, int fd_)
 {
-    if (monitor_events & ZMQ_EVENT_ACCEPTED) {
-        zmq_event_t event;
-        event.event = ZMQ_EVENT_ACCEPTED;
-        event.value = fd_;
-        monitor_event (event, addr_);
-    }
+    if (monitor_events & ZMQ_EVENT_LISTENING)
+        monitor_event (ZMQ_EVENT_LISTENING, fd_, addr_);
 }
 
-void zmq::socket_base_t::event_accept_failed (std::string &addr_, int err_)
+void zmq::socket_base_t::event_bind_failed (const std::string &addr_, int err_)
 {
-    if (monitor_events & ZMQ_EVENT_ACCEPT_FAILED) {
-        zmq_event_t event;
-        event.event = ZMQ_EVENT_ACCEPT_FAILED;
-        event.value= err_;
-        monitor_event (event, addr_);
-    }
+    if (monitor_events & ZMQ_EVENT_BIND_FAILED)
+        monitor_event (ZMQ_EVENT_BIND_FAILED, err_, addr_);
 }
 
-void zmq::socket_base_t::event_closed (std::string &addr_, int fd_)
+void zmq::socket_base_t::event_accepted (const std::string &addr_, int fd_)
 {
-    if (monitor_events & ZMQ_EVENT_CLOSED) {
-        zmq_event_t event;
-        event.event = ZMQ_EVENT_CLOSED;
-        event.value = fd_;
-        monitor_event (event, addr_);
-    }
-}
-        
-void zmq::socket_base_t::event_close_failed (std::string &addr_, int err_)
-{
-    if (monitor_events & ZMQ_EVENT_CLOSE_FAILED) {
-        zmq_event_t event;
-        event.event = ZMQ_EVENT_CLOSE_FAILED;
-        event.value = err_;
-        monitor_event (event, addr_);
-    }
+    if (monitor_events & ZMQ_EVENT_ACCEPTED)
+        monitor_event (ZMQ_EVENT_ACCEPTED, fd_, addr_);
 }
 
-void zmq::socket_base_t::event_disconnected (std::string &addr_, int fd_)
+void zmq::socket_base_t::event_accept_failed (const std::string &addr_, int err_)
 {
-    if (monitor_events & ZMQ_EVENT_DISCONNECTED) {
-        zmq_event_t event;
-        event.event = ZMQ_EVENT_DISCONNECTED;
-        event.value = fd_;
-        monitor_event (event, addr_);
-    }
+    if (monitor_events & ZMQ_EVENT_ACCEPT_FAILED)
+        monitor_event (ZMQ_EVENT_ACCEPT_FAILED, err_, addr_);
 }
 
-void zmq::socket_base_t::monitor_event (zmq_event_t event_, const std::string& addr_)
+void zmq::socket_base_t::event_closed (const std::string &addr_, int fd_)
+{
+    if (monitor_events & ZMQ_EVENT_CLOSED)
+        monitor_event (ZMQ_EVENT_CLOSED, fd_, addr_);
+}
+
+void zmq::socket_base_t::event_close_failed (const std::string &addr_, int err_)
+{
+    if (monitor_events & ZMQ_EVENT_CLOSE_FAILED)
+        monitor_event (ZMQ_EVENT_CLOSE_FAILED, err_, addr_);
+}
+
+void zmq::socket_base_t::event_disconnected (const std::string &addr_, int fd_)
+{
+    if (monitor_events & ZMQ_EVENT_DISCONNECTED)
+        monitor_event (ZMQ_EVENT_DISCONNECTED, fd_, addr_);
+}
+
+//  Send a monitor event
+void zmq::socket_base_t::monitor_event (int event_, int value_, const std::string &addr_)
 {
     if (monitor_socket) {
-        const uint16_t eid = (uint16_t)event_.event;
-        const uint32_t value = (uint32_t)event_.value;
-        // prepare and send first message frame
-        // containing event id and value
+        //  Send event in first frame
         zmq_msg_t msg;
-        zmq_msg_init_size (&msg, sizeof(eid) + sizeof(value));
-        char* data1 = (char*)zmq_msg_data(&msg);
-        memcpy (data1, &eid, sizeof(eid));
-        memcpy (data1+sizeof(eid), &value, sizeof(value));
+        zmq_msg_init_size (&msg, 6);
+        uint8_t *data = (uint8_t *) zmq_msg_data (&msg);
+        *(uint16_t *) (data + 0) = (uint16_t) event_;
+        *(uint32_t *) (data + 2) = (uint32_t) value_;
         zmq_sendmsg (monitor_socket, &msg, ZMQ_SNDMORE);
-        // prepare and send second message frame
-        // containing the address (endpoint)
+
+        //  Send address in second frame
         zmq_msg_init_size (&msg, addr_.size());
-        memcpy(zmq_msg_data(&msg), addr_.c_str(), addr_.size());
+        memcpy (zmq_msg_data (&msg), addr_.c_str (), addr_.size ());
         zmq_sendmsg (monitor_socket, &msg, 0);
     }
 }
 
-void zmq::socket_base_t::stop_monitor()
+void zmq::socket_base_t::stop_monitor (void)
 {
     if (monitor_socket) {
-        if (monitor_events & ZMQ_EVENT_MONITOR_STOPPED) {
-            zmq_event_t event;
-            event.event = ZMQ_EVENT_MONITOR_STOPPED;
-            event.value = 0;
-            monitor_event (event, "");
-        }
+        if (monitor_events & ZMQ_EVENT_MONITOR_STOPPED)
+            monitor_event (ZMQ_EVENT_MONITOR_STOPPED, 0, "");
         zmq_close (monitor_socket);
         monitor_socket = NULL;
         monitor_events = 0;
