@@ -57,7 +57,7 @@ def main():
     rep.bind('inproc://a')
     req.connect('inproc://a')
     A = numpy.ones((1024,1024))
-    print ("Array is %i bytes" % (len(A) * 8))
+    print ("Array is %i bytes" % (A.size * A.itemsize))
     
     # send/recv with pickle+zip
     req.send_zipped_pickle(A)
@@ -71,4 +71,4 @@ def main():
     print ("Okay" if (C==B).all() else "Failed")
 
 if __name__ == '__main__':
-    main()
\ No newline at end of file
+    main()
